# frozen_string_literal: true

module  Rspeckled
module  Faraday
module  Middleware
class   CurlLogger < ::Faraday::Response::Middleware
  def self.default_logger
    require 'logger'

    ::Logger.new(STDOUT)
  end

  def initialize(app, logger = nil, program_name = nil)
    super(app)

    @logger       = logger || self.class.default_logger
    @program_name = program_name
  end

  def call(env)
    @logger.info(@program_name) { curl_output('request', "#{env[:method].upcase} #{env[:url]}", env[:request_headers], env[:body]) }

    super
  end

  def on_complete(env)
    @logger.info(@program_name) { curl_output('response', "HTTP #{env[:status]}", env[:response_headers], env[:body]) }
  end

  private

  def curl_output(direction, summary, headers, body)
    directional_arrow = (direction == 'request') ? '>' : '<'
    color_code        = (direction == 'request') ? 32 : 33

    curl_formatted_headers = headers.map { |name, value|
      colorize("#{directional_arrow} #{name}: #{value}", color_code)
    }.join("\n")

    [
      '',
      colorize("#{directional_arrow} #{summary}", color_code),
      curl_formatted_headers,
      '',
      colorize(body, 37),
    ].join("\n")
  end

  def colorize(string, color_code)
    "\e[#{color_code}m#{string}\e[0m"
  end
end
end
end
end
