# frozen_string_literal: true

RSpec.configure do |config|
  config.before(:all, :elasticsearch => lambda { |v| !!v }) do
    config.rspeckled_logger.debug("Before All   - Start - #{__FILE__}")

    ::Elasticsearch::Extensions::Test::Cluster.start(:port => 9200) unless ::Elasticsearch::Extensions::Test::Cluster.running?

    config.rspeckled_logger.debug("Before All   - End   - #{__FILE__}")
  end

  config.around(:each, :elasticsearch => lambda { |v| !!v }) do |example|
    config.rspeckled_logger.debug("Around Each  - Start - #{__FILE__}")

    model = case example.metadata[:elasticsearch]
            when ::String
              example.metadata[:elasticsearch].constantize
            end

    model.__elasticsearch__.client.indices.delete :index => '_all'
    model.__elasticsearch__.create_index!         :index => model.index_name

    example.run

    model.__elasticsearch__.client.indices.delete :index => '_all'

    config.rspeckled_logger.debug("Around Each  - End   - #{__FILE__}")
  end

  config.after(:all, :elasticsearch => lambda { |v| !!v }) do
    config.rspeckled_logger.debug("After  All   - Start - #{__FILE__}")

    ::Elasticsearch::Extensions::Test::Cluster.stop(:port => 9200) if ::Elasticsearch::Extensions::Test::Cluster.running?

    config.rspeckled_logger.debug("After  All   - End   - #{__FILE__}")
  end
end
